--  ============================================================================
--
--  Toplevel of FPGA.
--
--  Instantiates example Amba based subsystem generated by CoreConsole.
--
--  ============================================================================

--------------------------------------------------------------------------------
--  The user must create this toplevel file which instantiates the CoreConsole
--  generated subsystem.
--  Any bi-directional signals must be handled at this level, as CoreConsole
--  currently is not capable of dealing with tri-states.
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity TutorialTop is
    port(
        SYSCLK          : in  std_logic;                        -- External Clock Input
        NSYSRESET       : in  std_logic;                        -- External Reset Input

        RemapDefault    : in  std_logic;
        HIGH            : out std_logic;
        LOW             : out std_logic;

        FLASH_BYTEN     : out std_logic;                        -- Flash Byte Mode Select
        FLASH_CSN       : out std_logic;                        -- Flash Chip Select
        FLASH_OEN       : out std_logic;                        -- Flash Chip Output Enable
        FLASH_RPN       : out std_logic;                        -- Flash Reset/Block Temporary Unprotect
        FLASH_WEN       : out std_logic;                        -- Flash Chip Byte Write Enable

        SRAM_ADSC       : out std_logic;                        -- SRAM Chip Address Strobe (Cache)
        SRAM_ADSP       : out std_logic;                        -- SRAM Chip Address Strobe (Processor)
        SRAM_ADV        : out std_logic;                        -- SRAM Chip Burst Address Enable
        SRAM_BYTE_WEN   : out std_logic;                        -- SRAM Chip Byte Write Enable
        SRAM_BYTEN      : out std_logic_vector(3 downto 0);     -- SRAM Chip Byte 0..3 Enable
        SRAM_CLK        : out std_logic;                        -- SRAM Chip Clock
        SRAM_CSN        : out std_logic;                        -- SRAM Chip Select
        SRAM_GLOBAL_WEN : out std_logic;                        -- SRAM Chip Global Write Enable
        SRAM_OEN        : out std_logic;                        -- SRAM Chip Output Enable
        SRAM_PWRDWN     : out std_logic;                        -- SRAM Chip Power-down Enable

        MEM_ADDR        : out std_logic_vector(18 downto 0);    -- Flash/SRAM Address Bus
        MEM_DATA        : inout std_logic_vector(31 downto 0);  -- Flash/SRAM Data Bus

        SW              : in  std_logic_vector(7 downto 0);     -- Switch Inputs
        LED             : out std_logic_vector(7 downto 0);     -- LED Outputs

        RX0             : in  std_logic;                        -- UART0 Receive
        TX0             : out std_logic;                        -- UART0 Transmit

        ICE_nTRST        : in  std_logic;                       -- RealView ICE JTAG Reset
        ICE_TCK          : in  std_logic;                       -- RealView ICE JTAG Clock Enable
        ICE_TDI          : in  std_logic;                       -- RealView ICE JTAG Data In
        ICE_TMS          : in  std_logic;                       -- RealView ICE JTAG Mode Select
        ICE_VTref        : out std_logic;                       -- RealView ICE Target Reference Voltage
        ICE_TDO          : out std_logic;                       -- RealView ICE JTAG Data Out
        ICE_RTCK         : out std_logic;                       -- RealView ICE RTCK (Used for Adaptive Clocking)
        ICE_nSRST        : inout std_logic;                     -- RealView ICE JTAG System Reset
        ICE_DBGACK       : out std_logic;                       -- RealView ICE Debug Acknowledge (Not Used)
        ICE_DBGRQ        : out std_logic                        -- RealView ICE Debut Request (Not Used)
    );
end TutorialTop;

architecture structural of TutorialTop is

--------------------------------------------------------------------------------
-- Component declarations
--------------------------------------------------------------------------------
    component TutorialMP7
    port(
        HCLK               : out std_logic;
        Mem_FlashCSN    : out std_logic;
        Mem_FlashOEnN   : out std_logic;
        Mem_FlashWEnN   : out std_logic;
        Mem_MemAddr        : out std_logic_vector(27 downto 0);
        Mem_MemDataIn      : in  std_logic_vector(31 downto 0);
        Mem_MemDataOEnN    : out std_logic;
        Mem_MemDataOut     : out std_logic_vector(31 downto 0);
        Mem_MemReadN       : out std_logic;
        Mem_MemWriteN      : out std_logic;
        Mem_SramByte0N  : out std_logic;
        Mem_SramByte1N  : out std_logic;
        Mem_SramByte2N  : out std_logic;
        Mem_SramByte3N  : out std_logic;
        Mem_SramCSN     : out std_logic;
        Mem_SramOEnN    : out std_logic;
        Mem_SramWEnN    : out std_logic;
        NSYSRESET          : in  std_logic;
        RV_RV_RTCK         : out std_logic;
        RV_RV_TCK          : in  std_logic;
        RV_RV_TDI          : in  std_logic;
        RV_RV_TDOUT        : out std_logic;
        RV_RV_TMS          : in  std_logic;
        RV_RV_nSRST_IN     : in  std_logic;
        RV_RV_nTDOEN       : out std_logic;
        RV_RV_nTRST        : in  std_logic;
        ReMapDef           : in  std_logic;
        SYSCLK             : in  std_logic;
        UART_RX            : in  std_logic;
        UART_TX            : out std_logic;
        keyPadIn           : in  std_logic_vector(31 downto 0);
        ledOut             : out std_logic_vector(31 downto 0)
    );
    end component;

--------------------------------------------------------------------------------
-- Signal declarations
--------------------------------------------------------------------------------

    signal MEM_DATA_IN  : std_logic_vector(31 downto 0);        -- Flash/SRAM Data Bus Input
    signal MEM_DATA_OUT : std_logic_vector(31 downto 0);        -- Flash/SRAM Data Bus Output
    signal MEM_DATA_OEN : std_logic;                            -- Flash/SRAM Data Bus Output Enable
    signal MEM_ADDR_INT : std_logic_vector(27 downto 0);        -- Flash/SRAM Address Bus (Internal)

    signal ICE_nSRST_IN  : std_logic;                           -- RealView ICE System Reset Input
    signal ICE_TDOUT     : std_logic;                           -- RealView ICE Test Data Out
    signal ICE_nTDOEN    : std_logic;                           -- RealView ICE Test Data Out Enable

    signal keyPadIn      : std_logic_vector(31 downto 0);       -- CoreMP7 DevKit Board Switches
    signal ledOut        : std_logic_vector(31 downto 0);       -- CoreMP7 DevKit Board LEDs
    
    signal HCLK          : std_logic;                           -- CoreMP7 Synchronized Clock
    signal SYSCLK_DIV4   : std_logic;                           -- CoreMP7 System Clock (Divided SYSCLK)
    signal iCOUNT        : std_logic_vector( 1 downto 0);

--------------------------------------------------------------------------------
-- Beginning of main code
--------------------------------------------------------------------------------
begin

--------------------------------------------------------------------------------
-- Fixed memory outputs
--------------------------------------------------------------------------------

    -- Tie all the fixed FLASH outputs to appropriate levels
    FLASH_BYTEN         <= '1';         -- Flash byte mode enable (16-bit mode)
    FLASH_RPN           <= NSYSRESET;   -- Flash reset/block temporary unprotect
    
    SRAM_CLK            <= NOT HCLK;    -- SRAM Clock Input Signal
    SRAM_CSN            <= '0';
    SRAM_PWRDWN         <= 'Z';         -- SRAM Sleep Mode Control (On-chip Pull-down, Active High)
    SRAM_ADV            <= '1';         -- SRAM Burst Address Counter Enable (Active Low)
    SRAM_ADSP           <= '1';         -- SRAM Address Strobe Processor (Active Low)
    SRAM_ADSC           <= '0';         -- SRAM Address Strobe Controller (Active Low)
    SRAM_GLOBAL_WEN     <= '1';         -- SRAM Global Write Enable (Active Low)
                                        --   use Byte Write Enable to Control Writes

--------------------------------------------------------------------------------
-- Signal assignments
--------------------------------------------------------------------------------
    SYSCLK_DIV4 <= iCOUNT(1);

    -- HIGH and LOW outputs are intended to enable setting of RemapDefault input
    -- externally using a jumper.
    -- HIGH, RemapDefault & LOW will be connected to adjacent header pins on board.
    HIGH <= '1';
    LOW  <= '0';

    -- When RV_nTDOEN is low, data is being driven on RV_TDOUT
    ICE_TDO <= ICE_TDOUT when (ICE_nTDOEN = '0') else 'Z';

    -- Generate the bi-directional RV_nSRST signal buffer.
    -- Note that this has to act like an open-collector output, so it only
    -- ever drives out a low signal. Otherwise, it goes high-impedance,
    -- and is pulled high by an external resistor.
    ICE_nSRST <= '0' when (NSYSRESET = '0') else 'Z';
    ICE_nSRST_IN <= ICE_nSRST;

    -- Tie RealView target reference voltage signal high. This signal indicates to
    -- that the target has power and must be at least 0.628 V.
    ICE_VTref <= '1';

    -- Tie unused RealView signals low.
    ICE_DBGACK <= '0';
    ICE_DBGRQ  <= '0';

    keyPadIn (31 downto 8) <= "000000000000000000000000";
    keyPadIn ( 7 downto 0) <= SW;
    
    LED <= ledOut ( 7 downto 0);
    

    -- Memory address bus
    MEM_ADDR <= MEM_ADDR_INT(18 downto 0);

    -- Bi-directional memory data bus
    MEM_DATA <= MEM_DATA_OUT when (MEM_DATA_OEN = '0') else "ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ";
    MEM_DATA_IN <= MEM_DATA;

--------------------------------------------------------------------------------
-- Instantiation of subsystem
--------------------------------------------------------------------------------

    TutorialTop : TutorialMP7
        port map(
            HCLK               => HCLK,
            Mem_FlashCSN       => FLASH_CSN,
            Mem_FlashOEnN      => FLASH_OEN,
            Mem_FlashWEnN      => FLASH_WEN,
            Mem_MemAddr        => MEM_ADDR_INT,
            Mem_MemDataIn      => MEM_DATA_IN,
            Mem_MemDataOEnN    => MEM_DATA_OEN,
            Mem_MemDataOut     => MEM_DATA_OUT,
            Mem_MemReadN       => OPEN,
            Mem_MemWriteN      => OPEN,
            
            Mem_SramByte0N     => SRAM_BYTEN(0),
            Mem_SramByte1N     => SRAM_BYTEN(1),
            Mem_SramByte2N     => SRAM_BYTEN(2),
            Mem_SramByte3N     => SRAM_BYTEN(3),
            Mem_SramWEnN       => SRAM_BYTE_WEN,

            Mem_SramCSN        => OPEN,
            Mem_SramOEnN       => SRAM_OEN,
            NSYSRESET          => NSYSRESET,
            RV_RV_RTCK         => ICE_RTCK,
            RV_RV_TCK          => ICE_TCK,
            RV_RV_TDI          => ICE_TDI,
            RV_RV_TDOUT        => ICE_TDOUT,
            RV_RV_TMS          => ICE_TMS,
            RV_RV_nSRST_IN     => ICE_nSRST_IN,
            RV_RV_nTDOEN       => ICE_nTDOEN,
            RV_RV_nTRST        => ICE_nTRST,
            ReMapDef           => RemapDefault,
            SYSCLK             => SYSCLK_DIV4,
            UART_RX            => RX0,
            UART_TX            => TX0,
            keyPadIn           => keyPadIn,
            ledOut             => ledOut
        );

--------------------------------------------------------------------------------
-- Divide external clock input by 4
--------------------------------------------------------------------------------
    process (SYSCLK, NSYSRESET)
    begin
        if (NSYSRESET = '0') then
            iCOUNT <= "00";
        elsif (SYSCLK'event and SYSCLK = '1') then
            iCOUNT <= iCOUNT + '1';
        end if;
    end process;

end structural;

--================================= End ===================================--
