
library ieee;
use ieee.std_logic_1164.all;

entity testbench is
end testbench;

architecture behavioral of testbench is

    constant SYSCLK_PERIOD : time := 100 ns;

    -- Component to test
    component TutorialTop
        -- Port list
        port(
            SYSCLK          : in    std_logic;
            NSYSRESET       : in    std_logic;
            RemapDefault    : in    std_logic;
            HIGH            :   out std_logic;
            LOW             :   out std_logic;


            FLASH_BYTEN     :   out std_logic;
            FLASH_CSN       :   out std_logic;
            FLASH_OEN       :   out std_logic;
--            FLASH_RPN       :   out std_logic;
            FLASH_WEN       :   out std_logic;

            SRAM_ADSC       :   out std_logic;
            SRAM_ADSP       :   out std_logic;
            SRAM_ADV        :   out std_logic;
            SRAM_BYTE_WEN   :   out std_logic;
            SRAM_BYTEN      :   out std_logic_vector(3 downto 0);
            SRAM_CLK        :   out std_logic;
            SRAM_CSN        :   out std_logic;
            SRAM_GLOBAL_WEN :   out std_logic;
            SRAM_OEN        :   out std_logic;
            SRAM_PWRDWN     :   out std_logic;

            MEM_ADDR        :   out std_logic_vector(18 downto 0);
            MEM_DATA        : inout std_logic_vector(31 downto 0);

            SW              : in    std_logic_vector(7 downto 0);
            LED             :   out std_logic_vector(7 downto 0);

            RX0             : in    std_logic;
            TX0             :   out std_logic;

            ICE_nTRST       : in    std_logic;
            ICE_TCK         : in    std_logic;
            ICE_TDI         : in    std_logic;
            ICE_TMS         : in    std_logic;
            ICE_VTref       :   out std_logic;
            ICE_TDO         :   out std_logic;
            ICE_RTCK        :   out std_logic;
            ICE_nSRST       : inout std_logic;
            ICE_DBGACK      :   out std_logic;
            ICE_DBGRQ       :   out std_logic 
        );
    end component;

    signal SYSCLK    : std_logic := '0';
    signal NSYSRESET : std_logic;
    signal ICE_nSRST : std_logic;

begin

    process
        variable vhdl_initial : BOOLEAN := TRUE; 

    begin
        if (vhdl_initial) then
            -- Assert system reset
            NSYSRESET <= '0';
            wait for (SYSCLK_PERIOD * 4);

            -- Release system reset
            NSYSRESET <= '1';
            wait;
        else
            wait;
        end if; 
    end process;

    -- SYSCLK signal
    SYSCLK <= not SYSCLK after (SYSCLK_PERIOD / 2);

    
    -- ICE_nSRST signal
    ICE_nSRST <= '0' when (NSYSRESET = '0') else 'H';


    -- Instantiate module to test
    TutorialTop_0 : TutorialTop
        -- Port map
        port map(
                SYSCLK          => SYSCLK,
                NSYSRESET       => NSYSRESET,
                
                RemapDefault    => '0',
                HIGH            => OPEN,
                LOW             => OPEN,
                
                FLASH_BYTEN     => OPEN,
                FLASH_CSN       => OPEN,
                FLASH_OEN       => OPEN,
--                FLASH_RPN       => OPEN,
                FLASH_WEN       => OPEN,
                
                SRAM_ADSC       => OPEN,
                SRAM_ADSP       => OPEN,
                SRAM_ADV        => OPEN,
                SRAM_BYTE_WEN   => OPEN,
                SRAM_BYTEN      => OPEN,
                SRAM_CLK        => OPEN,
                SRAM_CSN        => OPEN,
                SRAM_GLOBAL_WEN => OPEN,
                SRAM_OEN        => OPEN,
                SRAM_PWRDWN     => OPEN,
                
                MEM_ADDR        => OPEN,
                MEM_DATA        => OPEN,
                
                SW              => (others => '0'),
                LED             => OPEN,
                
                RX0             => '1',
                TX0             => OPEN,
                
                ICE_nTRST       => '1',
                ICE_TCK         => '0',
                ICE_TDI         => '0',
                ICE_TMS         => '1',
                ICE_TDO         => OPEN,
                ICE_RTCK        => OPEN,
                ICE_nSRST       => ICE_nSRST,
                ICE_DBGACK      => OPEN,
                ICE_DBGRQ       => OPEN
        );

end behavioral;
