;;  Copyright ARM Ltd 2002. All rights reserved.
;;
;;  This code initialises the Integrator board (eg REMAP) before calling
;;  TCM Initialization and MMU Initialization if they exist.
;;  this allows scatter loading to relocate code into the TCMs
;;
;;  This code must be run from a privileged mode

    PRESERVE8

; --- Standard definitions of mode bits and interrupt (I & F) flags in PSRs
Mode_USR        EQU     0x10
Mode_FIQ        EQU     0x11
Mode_IRQ        EQU     0x12
Mode_SVC        EQU     0x13
Mode_ABT        EQU     0x17
Mode_UND        EQU     0x1B
Mode_SYS        EQU     0x1F ; available on ARM Arch 4 and later

I_Bit           EQU     0x80 ; when I bit is set, IRQ is disabled
F_Bit           EQU     0x40 ; when F bit is set, FIQ is disabled


;IMPORT ||Image$$RAM_EXEC$$ZI$$Limit||
;TOP_OF_MEMORY  EQU    ||Image$$RAM_EXEC$$ZI$$Limit||

;TOP_OF_MEMORY   EQU 0x00480000    ;; Luas (Platform8051 board)
TOP_OF_MEMORY   EQU 0x101FFFFF    ;; CoreMP7 Dev. Kit Reversi demo

IRQ_STACK   EQU    TOP_OF_MEMORY
FIQ_STACK   EQU    IRQ_STACK-1024
SVC_STACK   EQU    FIQ_STACK-1024
USR_STACK   EQU    SVC_STACK-1024


        AREA   INIT, CODE, READONLY   ; name this block of code


        ENTRY

    IMPORT ||Image$$RAM_EXEC$$ZI$$Limit||


        EXPORT  Reset_Handler
Reset_Handler   FUNCTION



; Branch to C Library entry point

        IMPORT  __main
        BL      Setup_Stacks
        LDR     r12,=__main
        BX      r12                         ; branch to __main
        ENDFUNC

        IMPORT ||Image$$RAM_EXEC$$ZI$$Limit||
        EXPORT __user_initial_stackheap

heap_base       DCD     ||Image$$RAM_EXEC$$ZI$$Limit||

__user_initial_stackheap FUNCTION
    LDR   r0,heap_base
    MOV   pc,lr
    ENDFUNC



Setup_Stacks
        LDR     R0, =0xDEADBEEF
        LDR     R1, =USR_STACK
        STR     R0, [R1]

        MOV     R0, #Mode_IRQ:OR:I_Bit:OR:F_Bit
        MSR     CPSR_c, R0
        LDR     R13, =IRQ_STACK

        MOV     R0, #Mode_FIQ:OR:I_Bit:OR:F_Bit
        MSR     CPSR_c, R0
        LDR     R13, =FIQ_STACK
        LDR     R0, =0xDEADBEEF
        STR     R0, [R13]

        MOV     R0, #Mode_SVC:OR:I_Bit:OR:F_Bit
        MSR     CPSR_c, R0
        LDR     R13, =SVC_STACK
        LDR     R0, =0xDEADBEEF
        STR     R0, [R13]
        MOV     PC, LR

        END                                 ; mark the end of this file
