#include <stdio.h>
#include <string.h>
#include "BoardView.h"

#ifndef NULL
#define NULL	0
#endif

BoardView::BoardView()
{
	m_game = 0;
	m_xSel = 0;
	m_ySel = 0;
	m_xPrevSel = 0;
	m_yPrevSel = 0;
	
	m_comChannel = NULL;

    sprintf(m_textMsg, "Computer uses white color, human plays black color");	
//	for (int i = 0; i < TEXT_MESSAGE_MAX_SIZE; i++)
//	{
//		m_textMsg[i] = 0;
//	}
}

void BoardView::AttachGame(Game *pGame)
{
	m_game = pGame;
}

void BoardView::RegisterComChannel(ComChannel *comChannel)
{
	m_comChannel = comChannel;
}

void BoardView::SetSelectedSquare(int x, int y)
{
	m_xPrevSel = m_xSel;
	m_yPrevSel = m_ySel;
	m_xSel = x;
	m_ySel = y;
}

void BoardView::GetSelectedSquare(int *x, int *y)
{
    *x = m_xSel;
    *y = m_ySel;
}

void BoardView::SetTextMessage(char *pText, int size)
{
	if (size > TEXT_MESSAGE_MAX_SIZE)
		size = TEXT_MESSAGE_MAX_SIZE;
		
	memcpy(m_textMsg, pText, size);
	UpdateView();
}

void BoardView::UpdateView()
{
	char board[64];

	if (m_game == 0)
	{	// No game attached to the view.
		return;
	}

	int i, j;
	char squareLetter;

	for (i = 1; i < 9; i++)
	{
		for (j = 1; j < 9; j++)
		{
			Color squareColor; 
			
			squareColor = m_game->color(j, i);
			switch(squareColor)
			{
			case Black:
				squareLetter = 'b';	//'*';	//'b';
				break;

			case White:
				squareLetter = 'w';	//'O';	//'w';
				break;

			default:
				squareLetter = '-';	//'e';
				break;
			}

			board[((i-1) * 8) + (j-1)] = squareLetter;
		}
	}

	//
	// display selected square
	//
	squareLetter = board[((m_ySel-1) * 8) + (m_xSel-1)];
	switch(squareLetter)
	{
	case 'b':
		squareLetter = 'B';
		break;

	case 'w':
		squareLetter = 'W';
		break;

	case '-':
		squareLetter = '+';
		break;
	}
	board[((m_ySel-1) * 8) + (m_xSel-1)] = squareLetter;

#if USE_SEMI_HOSTING_FOR_OUTPUT == 1


	//
	//
	//
	for (i = 0; i < 64; i++)
	{
		printf("%c", board[i]);
		if (((i+1) % 8) == 0) printf("\n");
	}
	printf("\n");
	//
	// Display number of blacks and whites
	//
	printf("\nblack: %d, white: %d\n", m_game->score(Black), m_game->score(White));
#endif // RUN_GAME_ENGINE_ON_HOST

	//
	// Send the current board state to the GUI, whatever it might be...
	//
	if (m_comChannel != NULL)
	{
		m_comChannel->SendToGUI('!');
		
		for (i = 0; i < 64; i++)
			m_comChannel->SendToGUI(board[i]);
	
    	//
    	// Send the text message for the status bar. Make sure we send the '/0'
    	// termination character to cause the message to be displayed.
    	//
    	if (m_textMsg[0] != 0)
    	{
    		i = 0;
    		while ( (m_textMsg[i] != 0) && (i < TEXT_MESSAGE_MAX_SIZE) ) 
    		{
    			m_comChannel->SendToGUI(m_textMsg[i++]);
    		}
    		m_comChannel->SendToGUI(0);
    	}
	}
}
