#include <stdio.h>
#include "Reversi.h"

Reversi::Reversi()
{
	m_gameOver = false;
	m_competitiveGame = true;
	m_engine.setStrength(4);
	
	m_game.newGame();
	m_boardView.AttachGame(&m_game);
	m_boardView.UpdateView();
}

Reversi::~Reversi()
{
}

void Reversi::SetSelectedSquare(int x, int y)
{
	m_boardView.SetSelectedSquare(x, y);
	m_boardView.UpdateView();
}

void Reversi::GetSelectedSquare(int *x, int *y)
{
    m_boardView.GetSelectedSquare(x, y);
}

//
// returns true if move was legal and computer can make the next move.
//
bool Reversi::RequestHumanMove(int x, int y)
{
	if (m_gameOver == true)
	{	//
		// Interpret Select/Enter as request to start a new game.
		//
		m_gameOver = false;
		m_game.newGame();
    	char msg[TEXT_MESSAGE_MAX_SIZE];
    	sprintf(msg, "White: %d   Black: %d", m_game.score(White), m_game.score(Black));
    	m_boardView.SetTextMessage(msg, TEXT_MESSAGE_MAX_SIZE);
		m_boardView.UpdateView();
	}
	else
	{
		Color humanColor = m_game.toMove();
		Move humanMove(humanColor, x, y);

		if (m_game.makeMove(humanMove))
		{
	    	char msg[TEXT_MESSAGE_MAX_SIZE];
	    	sprintf(msg, "White: %d   Black: %d, ARM7 thinking", m_game.score(White), m_game.score(Black));
	    	m_boardView.SetTextMessage(msg, TEXT_MESSAGE_MAX_SIZE);
		}
		else
		{
	    	char msg[TEXT_MESSAGE_MAX_SIZE];
	    	sprintf(msg, "White: %d   Black: %d, Invalid move", m_game.score(White), m_game.score(Black));
	    	m_boardView.SetTextMessage(msg, TEXT_MESSAGE_MAX_SIZE);
		}

		if (!m_game.moveIsAtAllPossible()) 
		{
			m_gameOver = true;
			DisplayGameOver();
		}

		m_boardView.UpdateView();

		if (humanMove.color() != m_game.toMove())
		{
			return(true);
		}
	}
	return(false);
}

void Reversi::RequestComputerMove()
{
	ComputerMakeMove();
}

bool Reversi::IsGameOver()
{
	return(m_gameOver);
}

void Reversi::RegisterComChannel(ComChannel *comChannel)
{
	m_boardView.RegisterComChannel(comChannel);
}

/*	
void Reversi::HumanMakeMove(Move move)
{
	// Create a move from the mouse click and see if it is legal.
	// If it is, then make a human move.
	if (m_game.moveIsLegal(move)) 
	{
		m_game.makeMove(move);

		if (!m_game.moveIsAtAllPossible()) 
		{
			m_gameOver = true;
			DisplayGameOver();
			return;
		}

		m_boardView.UpdateView();

		if (move.color() != m_game.toMove())
			ComputerMakeMove();
	} 
}
*/

void Reversi::ComputerMakeMove()
{
	// Check if the computer can move.
	Color color    = m_game.toMove();
	Color opponent = ::opponent(color);

  	if (!m_game.moveIsPossible(color))
    	return;
 
  	// Make computer moves until the human can play or until the game is over.
  	do {
    	Move  move;

    	if (!m_game.moveIsAtAllPossible()) {
			m_gameOver = true;
			DisplayGameOver();
      		return;
    	}

		move = m_engine.computeMove(&m_game, m_competitiveGame);
    	if (move.x() == -1) {
      		return;
    	}

        m_boardView.SetSelectedSquare(move.x(), move.y());
    	m_game.makeMove(move);
    	char msg[TEXT_MESSAGE_MAX_SIZE];
    	sprintf(msg, "White: %d   Black: %d    your turn", m_game.score(White), m_game.score(Black));
    	m_boardView.SetTextMessage(msg, TEXT_MESSAGE_MAX_SIZE);
		m_boardView.UpdateView();
  	} while (!m_game.moveIsPossible(opponent));

 	if (!m_game.moveIsAtAllPossible()) 
  	{
	  	m_gameOver = true;
	  	DisplayGameOver();
		return;
  	}
}

void Reversi::DisplayGameOver()
{
	char msg[TEXT_MESSAGE_MAX_SIZE];
	
	sprintf(msg, "White: %d   Black: %d    Game Over", m_game.score(White), m_game.score(Black));
	
	m_boardView.SetTextMessage(msg, TEXT_MESSAGE_MAX_SIZE);
}

