#include "UARTComChannel.h"
#include "ReversiCfg.h"

#ifdef PLATFORM_8051    // Luas defines
    #define UART_BASE_ADDR   0x00D00000
    #define UART_STATUS_REG_OFFSET  0
    #define TX_REG_OFFSET           4

    // Status register bits
    #define SR_OVERFLOW     0x08
    #define SR_PARITY_ERROR 0x04
    #define SR_RXFULL       0x02
    #define SR_TXRDY        0x01

#else   // CoreMP7 Development Kit
    #define UART_BASE_ADDR          0xC3000000
    #define TX_REG_OFFSET           0X00
    #define UART_CTRL1_REG_OFFSET   0X08
    #define UART_CTRL2_REG_OFFSET   0X0C
    #define UART_STATUS_REG_OFFSET  0x10

    // Status register bits
    #define SR_OVERFLOW     0x08
    #define SR_PARITY_ERROR 0x04
    #define SR_RXFULL       0x02
    #define SR_TXRDY        0x01

    // Control register2 initialization values
    #define UART_CTRL2_n8 0x01  // no parity, 8bit data
    #define UART_CTRL2_e8 0x03  // even parity,8bit data
    #define UART_CTRL2_o8 0x07  // odd parity,8bit data

    // Baud rate constants
    // 10MHz clock
    #define baud10mhz_9600  0x41  // % Error
    #define baud10mhz_38400 0x10  // % Error

    // Baud rate constants
    //  12 MHz clock (48 MHz / 4)
    #define baud12mhz_9600  0x4D  // 0.16% Error
    #define baud12mhz_38400 0x12  // 2.80% Error

#endif


UARTComChannel::UARTComChannel()
{
	int i;
	for (i = 0; i < NB_BOARD_LOCATIONS; i++)
		m_boardState[i] = EMPTY;

	m_boardIdx = 0;
	m_textIdx = 0;

    #ifndef PLATFORM_8051
	// initialize UART control registers
//	*(unsigned char *)(UART_BASE_ADDR + UART_CTRL1_REG_OFFSET) = baud10mhz_9600;  //DevKitRev1
	*(unsigned char *)(UART_BASE_ADDR + UART_CTRL1_REG_OFFSET) = baud12mhz_9600;  //DevKitRev2
	*(unsigned char *)(UART_BASE_ADDR + UART_CTRL2_REG_OFFSET) = UART_CTRL2_e8;
    #endif

}

void UARTComChannel::SendToGUI(unsigned char txChar)
{
    volatile unsigned char *pTxReg;
    volatile unsigned char *pUARTStatus;

    pTxReg = (unsigned char *)(UART_BASE_ADDR + TX_REG_OFFSET);
    pUARTStatus = (unsigned char *)(UART_BASE_ADDR + UART_STATUS_REG_OFFSET);

    while ( !(*pUARTStatus & SR_TXRDY) )
    {
        ;
    }

    if (txChar == '\n')                    // Replace line feed with '\r'
        txChar = '\r';

    *pTxReg = txChar;

}

BoardLocationState UARTComChannel::GetLocation(int n)
{
	BoardLocationState location = EMPTY;

	if (n < NB_BOARD_LOCATIONS)
		location = m_boardState[n];

	return location;
}
