
#include <stdio.h>
#include "UserInput.h"
#include "ReversiCfg.h"

#if USE_SEMI_HOSTING_FOR_INPUT == 0
//#include "sysRegs.h"

    #ifdef PLATFORM_8051    // Luas defines
        #define SYS_REGS_BASE_ADDR  0x00C00000
        #define LED_STATE_REG_OFFSET        0
        #define KEYPAD_STATUS_REG_OFFSET    8

        #define LEDS_OFF    0xFF

        #define KSR_LEFT    0x10
        #define KSR_RIGHT   0x08
        #define KSR_UP      0x04
        #define KSR_DOWN    0x02
        #define KSR_HOME    0x01
    #else   // CoreMP7 Development Kit
        #define SYS_REGS_BASE_ADDR  0xC2000000
        #define LED_STATE_REG_OFFSET        0
        #define KEYPAD_STATUS_REG_OFFSET    0

        #define LEDS_OFF    0x00

        #define KSR_LEFT    0x02
        #define KSR_RIGHT   0x04
        #define KSR_UP      0x80
        #define KSR_DOWN    0x08
        #define KSR_HOME    0x01
    #endif

    #define DELAY_COUNTER_RELOAD 50000

#endif

#define INITIAL_X	2
#define INITIAL_Y	2
#define MAX_X		8
#define MIN_X		1
#define MAX_Y		MAX_X
#define MIN_Y		MIN_X

///////////////////////////////////////////////////////////////////////////////
void SwitchLedsOff(void)
{
    unsigned int *LedsCtrlReg;

    LedsCtrlReg = (unsigned int *)(SYS_REGS_BASE_ADDR + LED_STATE_REG_OFFSET);

    *LedsCtrlReg = LEDS_OFF;
}

void ToggleLED(void)
{
    static int ledState = 0;
    unsigned int *LedsCtrlReg;

    LedsCtrlReg = (unsigned int *)(SYS_REGS_BASE_ADDR + LED_STATE_REG_OFFSET);

    *LedsCtrlReg |= (ledState << 3);

    if (ledState > 0)
        ledState = 0;
    else
        ledState = 1;
}

void ShowHeartBeat(void)
{
    static int delayCounter = 0;

    if (delayCounter < 0)
    {
        delayCounter = DELAY_COUNTER_RELOAD;
        ToggleLED();
    }
    else
    {
        delayCounter--;
    }
}

///////////////////////////////////////////////////////////////////////////////
UserInput::UserInput()
{
	m_x = INITIAL_X;
	m_y = INITIAL_Y;
}

void UserInput::SetCurrentBoardLocation(int x, int y)
{
	if ((x <= MAX_X) && (x >= MIN_X))
		m_x = x;

	if ((y <= MAX_Y) && (y >= MAX_Y))
		m_y = y;
}

#define ENTER_KEY	0x73
#define UP_ARROW	0x75
#define DOWN_ARROW	0x64
#define RIGHT_ARROW	0x72
#define LEFT_ARROW	0x6C
#define QUIT_KEY	0x71


InputKey UserInput::GetKeyPress()
{
#if USE_SEMI_HOSTING_FOR_INPUT == 1
	InputKey key;
	int keyCode;
	bool validKey = false;

	while (!validKey)
	{
		keyCode = getchar();

		switch(keyCode)
		{
		case UP_ARROW:
			validKey = true;
			key = UP_KEY;
			break;

		case DOWN_ARROW:
			validKey = true;
			key = DOWN_KEY;
			break;

		case LEFT_ARROW:
			validKey = true;
			key = LEFT_KEY;
			break;

		case RIGHT_ARROW:
			validKey = true;
			key = RIGHT_KEY;
			break;

		case ENTER_KEY:
			validKey = true;
			key = SELECT_KEY;
			break;

		default:
			break;

		}
	}

	return(key);
#else
    InputKey key = UP_KEY;
    unsigned int *keypadStatus;
    unsigned int keypad;
    bool validKey = false;
    volatile int delay;

    keypadStatus = (unsigned int *)(SYS_REGS_BASE_ADDR + KEYPAD_STATUS_REG_OFFSET);

//    SwitchLedsOff();

    while (!validKey)
    {
        do
        {
            keypad = *keypadStatus;
            ShowHeartBeat();
        } while (keypad != 0);

        do
        {
            keypad = *keypadStatus;
            ShowHeartBeat();
        } while (keypad == 0);

        if (keypad & KSR_LEFT)
        {
            validKey = true;
            key = LEFT_KEY;
        }
        else if(keypad & KSR_RIGHT)
        {
            validKey = true;
            key = RIGHT_KEY;
        }
        else if(keypad & KSR_UP)
        {
            validKey = true;
            key = UP_KEY;
        }
        else if(keypad & KSR_DOWN)
        {
            validKey = true;
            key = DOWN_KEY;
        }
        else if(keypad & KSR_HOME)
        {
            validKey = true;
            key = SELECT_KEY;
        }
    }

    return(key);
#endif
}

/*
Move UserInput::GetNextMove(BoardView * view, Color humanColor)
{
	int key;
	bool bUpdateView = false;

	do {
//		key = getch();
		bUpdateView = false;
//		printf("/nkey: %x", key);

		switch(key)
		{
		case QUIT_KEY:
			// Indicate that the wishes to exit the app by setting an invalid
			// value for m_x.
			m_x = EXIT_REQUEST;
			break;

		case UP_ARROW:
			GoUp();
			bUpdateView = true;
			break;

		case DOWN_ARROW:
			GoDown();
			bUpdateView = true;
			break;

		case LEFT_ARROW:
			GoLeft();
			bUpdateView = true;
			break;

		case RIGHT_ARROW:
			GoRight();
			bUpdateView = true;
			break;

		case ENTER_KEY:
			// fall through
		default:
			break;
		}

		if (bUpdateView)
		{
			view->SetSelectedSquare(m_x, m_y);
			view->UpdateView();
		}
//		printf("\n x: %d, y: %d", m_x, m_y);

	} while ((key != ENTER_KEY) && (m_x != EXIT_REQUEST));

	Move nextMove(humanColor, m_x, m_y);

	return(nextMove);
}
*/
// private routines:

void UserInput::GoUp()
{
	if (m_y >  MIN_Y)
		m_y--;
}

void UserInput::GoDown()
{
	if (m_y < MAX_Y)
		m_y++;
}

void UserInput::GoRight()
{
	if (m_x < MAX_X)
		m_x++;
}

void UserInput::GoLeft()
{
	if (m_x > MIN_X)
		m_x--;
}
